import { Injectable } from '@angular/core';
import { NavigationEnd, Router, RouterEvent } from '@angular/router';
import { NbMediaBreakpointsService, NbMenuItem, NbThemeService } from '@nebular/theme';
import { combineLatest, of, forkJoin, ReplaySubject, empty, interval } from 'rxjs';
import { Observable } from 'rxjs/Observable';
import { catchError, debounce, debounceTime, delay, skip, takeUntil } from 'rxjs/operators';
import { map, tap } from 'rxjs/operators';
import { User } from '../models/user';
import { AuthService } from './auth.service';
import { BaseComponent } from '../../shared/components/base/base.component';
import { AppStateService } from './app.state.service';
import { CourseService } from 'app/features/courses/services/course.service';
import { Course } from 'app/features/courses/models/course';
import { Term } from 'app/features/modules/models/term';
import { environment } from 'environments/environment';
import { SubatoPrincipal } from '../models/principal/subato-principal';
import { TopMenu } from './top-menu';


@Injectable({
  providedIn: 'root'
})
export class PageMenuService extends BaseComponent {
  private _menu: ReplaySubject<NbMenuItem[]>
  menu$: Observable<NbMenuItem[]>

  menuSubscription = null;

  constructor(
    private authService: AuthService,
    private appStateService: AppStateService,
    private courseService: CourseService,
    private breakpointService: NbMediaBreakpointsService,
    private themeService: NbThemeService,
    router: Router) {
    super();

    this._menu = new ReplaySubject<NbMenuItem[]>();
    this.menu$ = this._menu.asObservable();

    combineLatest(router.events, this.authService.principal$, this.appStateService.term$, this.appStateService.courseId$, this.compactMenu())
      .pipe(takeUntil(this.unsubscribe))
      .subscribe(([val, principal, term, courseId, menuIsCompact]) => {
        //if (term == null) return;

        if ((<any>val).routerEvent instanceof NavigationEnd) {
          if (this.menuSubscription != null) {
            this.menuSubscription.unsubscribe();
          }

          this.menuSubscription = this.getMenu(principal, term, courseId, (<any>val).routerEvent.url, menuIsCompact)
            .pipe(takeUntil(this.unsubscribe))
            .subscribe(menu => {
              this._menu.next(menu);
            });
        }
      });
  }

  public getMenu(principal: SubatoPrincipal, term: Term | null, courseId: number | null, url: string, menuIsCompact: boolean): Observable<NbMenuItem[]> {
    let urlSegments = url.split('/');

    let menu: NbMenuItem[] = menuIsCompact 
      ? TopMenu.getMenu(this.appStateService.currentUrl, courseId, principal) 
      : [];

    if (courseId != null) {
      return this.courseService.getById(courseId).pipe(
        catchError(e => of(<Course>null)),
        tap(course => { 
          if(course.term != null) {
            this.appStateService.setTerm(course.term);
          } 
        }),
        map(course => course ? menu.concat(this.getCourseMenu(principal, course)) : menu)
      );
    } else {
      return of(menu);
    }
  }

  getCourseMenu(principal: SubatoPrincipal, course: Course): NbMenuItem[] {
    let courseBaseUrl = `/course/${course.id}`;
    let relativeUrl= this.appStateService.currentUrl.replace(courseBaseUrl, '');

    let menu: NbMenuItem[] = [
      {
        title: course.term ? `${course.shortName} (${course.term.name})` : course.shortName,
        group: true,
      },
      {
        title: 'Übersicht',
        icon: 'activity-outline',
        link: courseBaseUrl,
        selected: relativeUrl == ''
      },
      {
        title: 'Übungen',
        icon: 'file-text-outline',
        link: `${courseBaseUrl}/exercises`,
        selected: relativeUrl.startsWith('/exercises') || 
          relativeUrl.startsWith('/submissions') || 
          relativeUrl.startsWith('/solutions')
      },
      {
        title: 'Fortschritt',
        icon: 'bar-chart-outline',
        link: `${courseBaseUrl}/progress`,
        hidden: !(principal != null && principal.isParticipant(course.id)),
        selected: relativeUrl == '/progress'
      },
      {
        title: 'Veranstaltungen',
        icon: 'cube-outline',
        link: `${courseBaseUrl}/unit`,
        selected: relativeUrl.startsWith('/unit')
      },
      {
        title: 'Dateien',
        icon: 'file-outline',
        link: `${courseBaseUrl}/files`,
        selected: relativeUrl.startsWith('/files')
      },
      {
        title: 'Berechtigungen',
        icon: 'unlock-outline',
        link: `${courseBaseUrl}/permissions`,
        hidden: !(principal != null && principal.isAdmin(course.id)),
        selected: relativeUrl == '/permissions'
      },
      {
        title: 'Einstellungen',
        icon: 'settings-2-outline',
        link: `${courseBaseUrl}/settings`,
        hidden: !(principal != null && principal.isAdmin(course.id)),
        selected: relativeUrl == '/settings'
      },
      {
        icon: 'external-link-outline',
        title: 'Cardelli',
        url: environment.cardelliUrl,
        target: '_blank',
      },
    ];

    return menu;
  }

  compactMenu(): Observable<boolean> {
    const { lg } = this.breakpointService.getBreakpointsMap();
    return this.themeService.onMediaQueryChange()
      .pipe(
        map(([, currentBreakpoint]) => currentBreakpoint.width < lg),
        takeUntil(this.unsubscribe)
      );
  }
}