import { Component, Input, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { AuthService } from 'app/core/services/auth.service';
import { NotificationService } from 'app/shared/services/notification.service';
import { combineLatest, forkJoin, Subject } from 'rxjs';
import { debounceTime, filter, flatMap, map, takeUntil } from 'rxjs/operators';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { CourseUnitService } from '../../services/course-unit.service';
import { CourseUnit } from '../../models/course-unit';
import { FormArray, UntypedFormBuilder, UntypedFormGroup, Validators } from '@angular/forms';
import { User } from 'app/core/models/user';
import { BackendError } from 'app/shared/models/backend-error';
import { UserService } from 'app/core/services/user.service';
import { Page } from 'app/shared/models/page';
import { DataQueryParams } from 'app/shared/models/data-query-params';
import { DialogAddParticipantComponent } from '../../dialogs/dialog-add-participant/dialog-add-participant.component';
import { NbDialogService } from '@nebular/theme';

@Component({
  selector: 'ngx-course-unit-participants',
  templateUrl: './course-unit-participants.component.html',
  styleUrls: ['./course-unit-participants.component.scss']
})
export class CourseUnitParticipantsComponent extends BaseComponent implements OnInit {
  loading: boolean = true;
  @Input() unit: CourseUnit;
  participants: User[] = [];
  allParticipants: User[] = [];

  error: BackendError;
  query: string;
  private queryChanged: Subject<string> = new Subject<string>();

  constructor(
    private courseUnitService: CourseUnitService,
    private notificationService: NotificationService,
    private dialogService: NbDialogService,
    private route: ActivatedRoute,
    private authService: AuthService,
    private router: Router,
    private fb: UntypedFormBuilder
  ) {
    super();
  }

  ngOnInit(): void {
    this.courseUnitService.getParticipants(this.unit.id)
      .pipe(takeUntil(this.unsubscribe))
      .subscribe(participants => {
        this.allParticipants = participants;
        this.participants = this.allParticipants;
        this.loading = false;
      }, error => {
        if (BackendError.isBackendError(error)) {
          this.error = error;
          this.loading = false;
          this.notificationService.showError(
            error.format(
              "Fehler beim Laden der Teilnehmer",
            )
          );
        } else {
          throw error;
        }
      });

    this.queryChanged
      .pipe(
        takeUntil(this.unsubscribe),
        debounceTime(500),
      )
      .subscribe(query => {
        this.participants = this.allParticipants.filter(
          p => p.name.toLowerCase().indexOf(query.toLowerCase()) != -1 ||
               (p.loginName != null && p.loginName.toLowerCase().indexOf(query.toLowerCase()) != -1)
        );
      });
  }

  removeParticipant(participant: User) {
    this.courseUnitService.removeParticipant(this.unit.id, participant.id)
      .pipe(takeUntil(this.unsubscribe))
      .subscribe(() => {
        let idx = this.participants.indexOf(participant);
        if (idx != -1) {
          this.participants.splice(idx, 1);
        }

        this.notificationService.showSuccess("Teilnehmer erfolgreich entfernt");
      }, (error: any) => {
        if (BackendError.isBackendError(error)) {
          this.notificationService.showError(
            error.format(
              "Fehler bei Entfernung des Teilnehmers",
            )
          );
        } else {
          throw error;
        }
      });
  }

  addParticipant() {
    this.dialogService.open(DialogAddParticipantComponent, {
      context: { }
    }).onClose.pipe(
      takeUntil(this.unsubscribe),
      filter(result => result && result.user),
      flatMap(result => this.courseUnitService.addParticipant(this.unit.id, result.user.id).pipe(map(() => result.user)))
    ).subscribe(user => {
      this.participants.splice(0, 0, user);
      this.notificationService.showSuccess("Teilnehmer erfolgreich hinzugefügt");
    }, (error: any) => {
      if (BackendError.isBackendError(error)) {
        this.notificationService.showError(
          error.format(
            "Fehler beim Hinzufügen des Teilnehmers",
          )
        );
      } else {
        throw error;
      }
    });
  }

  onChange(query) {
    this.queryChanged.next(query);
  }

  cancel() {

  }
}
