import { Component, Input, OnInit } from '@angular/core';
import { UntypedFormGroup, UntypedFormBuilder, Validators, UntypedFormArray, AbstractControl, ValidationErrors, ValidatorFn } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { CreateExerciseDto, CreateTaskInstanceDto } from 'app/features/exercises/dto/create-exercise.dto';
import { Exercise } from 'app/features/exercises/models/exercise';
import { ExerciseService } from 'app/features/exercises/services/exercise.service';
import { NotificationService } from 'app/shared/services/notification.service';
import { takeUntil, filter } from 'rxjs/operators';
import { FormComponent } from 'app/shared/components/base/form.component';
import { Course } from 'app/features/courses/models/course';
import { Task } from 'app/features/tasks/models/task';
import { TaskInstance } from 'app/features/exercises/models/task-instance';
import { CdkDragDrop, moveItemInArray } from '@angular/cdk/drag-drop';
import { BackendError } from 'app/shared/models/backend-error';
import { NbDialogService } from '@nebular/theme';
import { DialogConfirmationComponent } from 'app/shared/dialogs/dialog-confirmation/dialog-confirmation.component';
import { TinyMCEHelper } from 'app/shared/helper/tiny-mce.helper';
import { of } from 'rxjs';
import { Iso8601Helper } from 'app/shared/helper/iso8601.helper';

export const reachablePointsTaskInstancesValidator: ValidatorFn = (control: AbstractControl): ValidationErrors | null => {
  const usingPoints = control.get('usingPoints').value;
  const taskInstances = control.get('taskInstances') as UntypedFormArray;

  return usingPoints && taskInstances.controls.some(c => !c.get('reachablePoints').value) ? { pointsMissing: true } : null;
};

@Component({
  selector: 'ngx-exercise-form',
  templateUrl: './exercise-form.component.html',
  styleUrls: ['./exercise-form.component.scss']
})
export class ExerciseFormComponent extends FormComponent implements OnInit {
  @Input() exercise: Exercise | null;
  @Input() course: Course | null;

  saving: boolean = false;
  excerciseForm: UntypedFormGroup;
  loading: boolean = true;
  error: BackendError;
  editorLoaded: boolean = false;

  editorConfig = TinyMCEHelper.editorConfig;
  taskInstances: TaskInstance[] = [];

  syntaxHighlightExample = `<pre class="language-java">...</pre>`;
  
  constructor(
    private exerciseService: ExerciseService,
    private dialogService: NbDialogService,
    private router: Router,
    private route: ActivatedRoute,
    private notificationService: NotificationService,
    private fb: UntypedFormBuilder) {
    super();
  }

  ngOnInit(): void {
    let loadTaskInstances = this.exercise != null ? 
      this.exerciseService.getTaskInstances(this.exercise.id)
      : of(<TaskInstance[]>[]);

    loadTaskInstances
    .pipe(takeUntil(this.unsubscribe))
    .subscribe(taskInstances => {
      this.createForm(this.exercise, taskInstances);
      this.loading = false;
    }, error => {
      if (BackendError.isBackendError(error)) {
        this.loading = false;
        this.error = error;

        this.notificationService.showError(
          error.format(
            "Fehler beim Laden der Aufgabeninstanzen",
          )
        );
      } else {
        throw error;
      }
    });
  }

  get usingPoints(): boolean {
    return this.excerciseForm.get('usingPoints').value;
  }

  get taskInstancesForm() {
    return this.excerciseForm.controls["taskInstances"] as UntypedFormArray;
  }

  createForm(exercise?: Exercise, taskInstances?: TaskInstance[]) {
    this.excerciseForm = this.fb.group({
      name: [exercise?.name ?? '', Validators.required],
      usingPoints: [exercise?.usingPoints ?? false, Validators.required],
      description: [exercise?.description ?? ''],
      summary: [exercise?.summary ?? ''],
      dueTo: [exercise?.dueTo ?? ''],
      visibleFrom: [exercise?.visibleFrom ?? ''],
      taskInstances: this.fb.array(taskInstances?.map(ti => this.createTaskInstanceForm(ti)) ?? [])
    }, { validators: reachablePointsTaskInstancesValidator });
  }

  createTaskInstanceForm(taskInstance: TaskInstance) {
    return this.fb.group({
      reachablePoints: [taskInstance.reachablePoints, [Validators.pattern("^[0-9]*$")]],
      task: taskInstance.task
    });
  }


  cancel() {
    this.router.navigate(['/', 'course', this.course.id, 'exercises']);
  }

  save(confirm?: boolean) {
    this.saving = true;

    let taskInstances = this.taskInstancesForm.controls.map((c, i) => {
      let values = c.value;
      return <CreateTaskInstanceDto>{
        orderNumber: i,
        reachablePoints: values.reachablePoints ? parseInt(values.reachablePoints) : null,
        taskId: values.task.id
      }
    });

    let dto: CreateExerciseDto = {
      courseId: this.course.id,
      name: this.excerciseForm.get('name').value,
      usingPoints: this.excerciseForm.get('usingPoints').value,
      description: this.excerciseForm.get('description').value,
      summary: this.excerciseForm.get('summary').value,
      dueTo: Iso8601Helper.toString(this.excerciseForm.get('dueTo').value),
      visibleFrom: Iso8601Helper.toString(this.excerciseForm.get('visibleFrom').value),
      taskInstances: taskInstances
    }

    if (this.exercise != null) {
      this.exerciseService.update(this.exercise.id, dto, confirm)
        .pipe(takeUntil(this.unsubscribe))
        .subscribe(exercise => {
          this.saving = false;
          this.notificationService.showSuccess("Übung erfolgreich aktualisiert");
          this.router.navigate(['course', this.course.id, 'exercises']);
        }, (error: any) => {
          this.saving = false;
          if (BackendError.isBackendError(error)) {
            if (error.isDangerousAction) {
              this.dialogService.open(DialogConfirmationComponent, {
                context: {
                  title: `Achtung`,
                  text: `${error.data['description']}. ${error.data['suggestion']}. Trotzdem speichern?`,
                  actionText: 'Speichern'
                }
              }).onClose.pipe(
                takeUntil(this.unsubscribe),
                filter(result => result && result.confirmed),
              ).subscribe(() => {
                this.save(true);
              });

              return;
            }

            if (error.isValidation) {
              this.errors = this.getValidationErrors(error);
            }

            this.notificationService.showError(
              error.format(
                "Fehler bei Aktualisierung der Übung",
              )
            );
          } else {
            throw error;
          }
        });
    } else {
      this.exerciseService.create(dto)
        .pipe(takeUntil(this.unsubscribe))
        .subscribe(exercise => {
          this.saving = false;
          this.notificationService.showSuccess("Übung erfolgreich aktualisiert");
          this.router.navigate(['course', exercise.course.id, 'exercises']);
        }, (error: any) => {
          this.saving = false;
          if (BackendError.isBackendError(error)) {
            if (error.isValidation) {
              this.errors = this.getValidationErrors(error);
            }

            this.notificationService.showError(
              error.format(
                "Fehler bei Erstellung der Übung",
              )
            );
          } else {
            throw error;
          }
        });
    }
  }

  selectedTasks(): Task[] {
    return this.taskInstancesForm.controls.map(group => group.get('task').value);
  }

  selectTask(task: Task) {
    let taskInstance: TaskInstance = new TaskInstance({
      task: task,
      reachablePoints: null
    });
    let taskInstanceForm = this.createTaskInstanceForm(taskInstance);
    this.taskInstancesForm.push(taskInstanceForm);
  }

  removeTask(taskInstanceForm: UntypedFormGroup) {
    this.taskInstancesForm.removeAt(this.taskInstancesForm.controls.indexOf(taskInstanceForm));
  }

  drop(event: CdkDragDrop<string[]>) {
    moveItemInArray(this.taskInstancesForm.controls, event.previousIndex, event.currentIndex);
  }
}
