import { HttpClient, HttpParams } from "@angular/common/http";
import { Injectable } from "@angular/core";
import { BaseService } from "app/shared/services/base.service";
import { environment } from "environments/environment";
import { Observable } from "rxjs";
import { catchError, map } from "rxjs/operators";
import { CreateExerciseDto } from "../dto/create-exercise.dto";
import { Exercise } from "../models/exercise";
import { Submission } from "../models/submission";
import { withCache, CacheBucket, HttpCacheManager } from '@ngneat/cashew';
import { BackendError } from "app/shared/models/backend-error";
import { ExerciseSubmissions } from "../models/exercise-submissions";
import { UnitSubmissionsProgress } from "../models/unit-submissions-progress";
import { SubmissionsProgress } from "../models/submissions-progress";
import { ExerciseProgress } from "../models/exercise-progress";
import { TaskInstance } from "../models/task-instance";
import { FileRef } from "app/features/tasks/models/file-ref";
import { DownloadService } from "app/shared/services/download.service";

@Injectable({
    providedIn: 'root'
})
export class ExerciseService extends BaseService {
    constructor(private http: HttpClient, private cacheManager: HttpCacheManager, private downloadService: DownloadService) {
        super();
    }

    getById(id: number): Observable<Exercise> {
        return this.http.get<Exercise>(`${environment.backendUrl}/exercise/${id}`, {
            context: withCache()
        }).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(exercise => new Exercise(exercise))
        );
    }

    getTaskInstance(exerciseId: number, taskId: number): Observable<TaskInstance> {
        return this.http.get<TaskInstance>(`${environment.backendUrl}/exercise/${exerciseId}/task/${taskId}`, {
            context: withCache()
        }).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(taskInstance => new TaskInstance(taskInstance))
        );
    }

    create(dto: CreateExerciseDto): Observable<Exercise> {
        return this.http.post<Exercise>(`${environment.backendUrl}/exercise`, dto).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(exercise => {
                this.cacheManager.clear();
                return new Exercise(exercise);
            })
        );
    }

    update(exerciseId: number, dto: CreateExerciseDto, confirmDangerousAction?: boolean): Observable<Exercise> {
        let headers = {};
        if(confirmDangerousAction) {
            headers['ConfirmDangerousAction'] = "true";
        }

        return this.http.put<Exercise>(`${environment.backendUrl}/exercise/${exerciseId}`, dto, {
            headers: headers
        }).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(exercise => {
                this.cacheManager.clear();
                return new Exercise(exercise);
            })
        );
    }

    getForCourse(courseId: number): Observable<Exercise[]> {
        return this.http.get<Exercise[]>(`${environment.backendUrl}/exercise?courseId=${courseId}`, {
            context: withCache()
        }).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(exercises => exercises.map(exercise => new Exercise(exercise)))
        )
    }


    getProgressForUser(courseId: number, userId: number): Observable<ExerciseProgress[]> {
        let params = new HttpParams();
        params = params.set('courseId', courseId);
        params = params.set('userId', userId);
        
        return this.http.get<ExerciseProgress[]>(`${environment.backendUrl}/exercise/progress`, {
            params: params    
        }).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(exerciseProgresses => exerciseProgresses.map(exerciseProgress => new ExerciseProgress(exerciseProgress)))
        )
    }

    getTaskInstanceFiles(exerciseId: number, taskId: number): Observable<FileRef[]> {
        return this.http.get<FileRef[]>(`${environment.backendUrl}/exercise/${exerciseId}/task/${taskId}/files`, {
            context: withCache()
        })
        .pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(files => files.map(f => new FileRef(f)))
        );
    }

    getFileUrl(exerciseId: number, taskId: number, ref: FileRef): string {
        return `${environment.backendUrl}/exercise/${exerciseId}/task/${taskId}/files/${ref.file.uuid}/download`;
    }

    downloadFile(exerciseId: number, taskId: number, file: FileRef): Observable<void> {
        return this.downloadService.downloadFile(this.getFileUrl(exerciseId, taskId, file), true);
    }

    getFile(exerciseId: number, taskId: number, ref: FileRef): Observable<string> {
        return this.http.get(this.getFileUrl(exerciseId, taskId, ref), { 
            responseType: 'text',
            context: withCache()
        }).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map((data: any) => data)
        );
    }

    getTaskInstances(exerciseId: number): Observable<TaskInstance[]> {
        return this.http.get<TaskInstance[]>(`${environment.backendUrl}/exercise/${exerciseId}/task`, {
            context: withCache()
        })
        .pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(taskInstances => taskInstances.map(ti => new TaskInstance(ti)))
        );
    }

    delete(id: number): Observable<void> {
        return this.http.delete<void>(`${environment.backendUrl}/exercise/${id}`)
            .pipe(
                catchError(e => {
                    throw BackendError.parse(e);
                }),
                map(() => this.cacheManager.clear())
            );
    }

    publishSolution(exerciseId: number, taskId: number, publish: boolean): Observable<void> {
        return this.http.put<void>(`${environment.backendUrl}/exercise/${exerciseId}/task/${taskId}/public`, {
            pub: publish
        }).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(() => this.cacheManager.clear())
        );
    }
}