import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { User } from 'app/core/models/user';
import { AuthService } from 'app/core/services/auth.service';
import { ParticipantAttendance } from 'app/features/attendances/dto/participant-attendance.dto';
import { AttendanceService } from 'app/features/attendances/services/attendance.service';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { NotificationService } from 'app/shared/services/notification.service';
import { combineLatest, forkJoin, of } from 'rxjs';
import { takeUntil, flatMap, map } from 'rxjs/operators';
import { CourseUnit } from '../../../courses/models/course-unit';
import { CourseUnitService } from '../../../courses/services/course-unit.service';
import { BackendError } from 'app/shared/models/backend-error';
import { ExerciseProgress } from 'app/features/exercises/models/exercise-progress';
import { ExerciseService } from 'app/features/exercises/services/exercise.service';
import { ErrorContainerConfig } from 'app/shared/components/error-container/error-container.component';
import { AppStateService } from 'app/core/services/app.state.service';
import { Breadcrumb } from 'app/shared/components/breadcrumbs/breadcrumbs.component';
import { SubatoPrincipal } from 'app/core/models/principal/subato-principal';

@Component({
  selector: 'ngx-course-unit-progress',
  templateUrl: './course-unit-progress.component.html',
  styleUrls: ['./course-unit-progress.component.scss']
})
export class CourseUnitProgressComponent extends BaseComponent implements OnInit {
  selectedUser: User;
  principal: SubatoPrincipal;
  loading: boolean = false;
  loadingProgress: boolean = false;
  courseUnit: CourseUnit;
  participantAttendance: ParticipantAttendance;
  exerciseProgresses: ExerciseProgress[] = [];

  loadingProgressError: BackendError;
  error: BackendError;
  errorConfig: ErrorContainerConfig = {
    notFound: {
      message: 'Diese Veranstaltung konnten wir leider nicht finden.'
    }
  };

  breadcrumbs: Breadcrumb[] = [];

  constructor(private exerciseService: ExerciseService,
    private appStateService: AppStateService,
    private courseUnitService: CourseUnitService,
    private attendanceService: AttendanceService,
    private notificationService: NotificationService,
    private authService: AuthService,
    private route: ActivatedRoute,
    private router: Router) {
    super();
  }

  public onAttach() {
    this.appStateService.setCourse(this.courseUnit?.course?.id);
  }

  buildBreadcrumbs(unit: CourseUnit) {
    this.breadcrumbs = [
      {
        id: 'courseUnits',
        params: {
          course_id: unit.course.id
        },
      },
      {
        id: 'courseUnit',
        params: {
          unit_id: unit.id
        },
        ctx: unit
      },
      {
        id: 'courseUnitProgress',
        params: {
          unit_id: unit.id
        }
      },
    ];
  }

  ngOnInit(): void {
    this.loading = true;
    combineLatest(this.route.paramMap, this.authService.principal$)
      .pipe(
        takeUntil(this.unsubscribe),
        flatMap(([params, principal]) => {
          this.principal = principal;
          let unitId = Number(params.get("unit_id"));
          return this.courseUnitService.getById(unitId);
        })
      )
      .subscribe(courseUnit => {
        this.courseUnit = courseUnit;
        this.appStateService.setCourse(courseUnit.course.id);
        this.buildBreadcrumbs(courseUnit);
        this.loading = false;
      }, (error: any) => {
        if (BackendError.isBackendError(error)) {
          this.error = error;
          this.loading = false;
        } else {
          throw error;
        }
      });
  }

  selectUser(user: User) {
    this.selectedUser = user;
    this.loadingProgress = true;

    return forkJoin(
      this.attendanceService.getAttendancesForUserAndUnit(this.courseUnit.id, user.id), 
      this.exerciseService.getProgressForUser(this.courseUnit.course.id, user.id)
    )
    .pipe(
      takeUntil(this.unsubscribe),
    )
    .subscribe(([participantAttendance, courseResult]) => {
      this.participantAttendance = participantAttendance;
      this.exerciseProgresses = courseResult;
      this.loadingProgress = false;
    }, (error) => {
      this.loadingProgress = false;
      this.loadingProgressError = error;
    });
  }
}
