import { Component, EventEmitter, Input, OnChanges, OnInit, Output, SimpleChanges } from '@angular/core';
import { Page } from 'app/shared/models/page';
import { Solution } from '../../models/solution';
import { SolutionService } from '../../../solutions/services/solution.service';
import { combineLatest, empty, EMPTY, forkJoin, interval, Observable, of } from 'rxjs';
import { takeUntil, expand, flatMap, delay } from 'rxjs/operators';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { AuthService } from 'app/core/services/auth.service';
import { DataQueryParams } from 'app/shared/models/data-query-params';
import { BackendError } from 'app/shared/models/backend-error';

@Component({
  selector: 'ngx-solution-history',
  templateUrl: './solution-history.component.html',
  styleUrls: ['./solution-history.component.scss']
})
export class SolutionHistoryComponent extends BaseComponent implements OnChanges {
  @Input() exerciseId: number;
  @Input() taskId: number;
  @Input() userId: number;
  @Input() submissionId: number;
  @Input() solution: Solution | null = null;
  @Input() style: string = "";
  @Input() showExternalLink: boolean = false;

  @Output() solutionSelected: EventEmitter<Solution> = new EventEmitter<Solution>();
  @Output() solutionsLoaded: EventEmitter<Solution[]> = new EventEmitter<Solution[]>();

  loading: boolean = true;
  error: BackendError;

  solutions: Solution[] = [];
  page: Page<Solution>;
  loadingPage: boolean;
  solutionQueryParams: DataQueryParams = { sortBy: 'submittedAt', sortOrder: 'desc' };

  constructor(
    private solutionService: SolutionService,
  ) {
    super();
  }

  ngOnChanges(changes: SimpleChanges) {
    var shouldRefresh = changes.hasOwnProperty('taskId') || 
    changes.hasOwnProperty('exerciseId') || 
    changes.hasOwnProperty('exerciseId') || 
    changes.hasOwnProperty('submissionId'); 

    if(shouldRefresh) {
      this.refresh();
    }
  }

  refresh() {
    this.loading = true;
    let loadSolutions = this.userId != null ? this.solutionService.getSolutions({
      exerciseId: this.exerciseId,
      taskId: this.taskId,
      userId: this.userId,
      submissionId: this.submissionId
    }, this.solutionQueryParams) : of(new Page<Solution>());
    
    loadSolutions.pipe(
        takeUntil(this.unsubscribe),
      )
      .subscribe(solutions => {
        this.solutions = solutions.content;
        this.solutionsLoaded.emit(this.solutions);
        this.page = solutions;

        this.loading = false;
      }, error => {
        if (BackendError.isBackendError(error)) {
          this.error = error;
          this.loading = false;
        } else {
          throw error;
        }
      })
  }

  selectSolution(solution: Solution) {
    this.solutionSelected.emit(solution);
  }

  loadNextSolutions(reset: boolean = false) {
    if (this.loadingPage || (this.page != null && !this.page.hasNext())) { return }
    this.loadingPage = true;

    let projection = DataQueryParams.next(this.page, this.solutionQueryParams);

    this.solutionService.getSolutions({
      exerciseId: this.exerciseId,
      taskId: this.taskId,
      userId: this.userId,
      submissionId: this.submissionId
    }, projection).subscribe(page => {
      this.page = page;
      if (reset) {
        this.solutions = [];
      }
      this.solutions.push(...page.content);
      this.solutionsLoaded.emit(this.solutions);
      this.loadingPage = false;
    }, error => {
      if (BackendError.isBackendError(error)) {
        this.error = error;
        this.loading = false;
      } else {
        throw error;
      }
    });
  }
}
