import { Component, Input, OnInit } from '@angular/core';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { FileTree } from 'app/shared/models/file-tree';
import { takeUntil } from 'rxjs/operators';
import { SolutionService } from '../../services/solution.service';
import { File } from 'app/shared/components/source-code-viewer/source-code-viewer.component';
import { NotificationService } from 'app/shared/services/notification.service';
import { DomSanitizer, SafeResourceUrl } from '@angular/platform-browser';
import { BackendError } from 'app/shared/models/backend-error';
import { Solution } from '../../models/solution';

enum FileViewer {
  sourceCode,
  pdf,
  none
}

@Component({
  selector: 'solution-viewer',
  templateUrl: './solution-viewer.component.html',
  styleUrls: ['./solution-viewer.component.scss']
})
export class SolutionViewerComponent extends BaseComponent implements OnInit {
  loading: boolean;
  currentTree: FileTree;
  currentFile: File;
  _solution: Solution;
  compactMode: boolean = false;

  FileViewer = FileViewer;
  pdfEmbedUrl: SafeResourceUrl;

  constructor(
    private solutionService: SolutionService,
    private notificationService: NotificationService,
    private sanitizer: DomSanitizer
  ) {
    super();
  }

  ngOnInit(): void {
  }

  @Input() set solution(val: Solution) {
    this._solution = val;
    if (this._solution.tree.children.length != 0) {
      let first = this._solution.tree.children[0]
      this.compactMode = val.tree.children.length == 1 && first.children.length == 0;
      this.onTreeSelected(first);
    }
  }

  get solution() {
    return this._solution;
  }

  get viewer(): FileViewer {
    if (this.currentTree == null) {
      return null;
    }

    if (this.currentTree.src) {
      return FileViewer.sourceCode;
    }

    if (!this.currentTree.src && this.currentTree.isPdf) {
      return FileViewer.pdf;
    }

    return FileViewer.none;
  }

  onTreeSelected(tree: FileTree) {
    if (tree.children.length != 0) {
      return;
    }
    
    this.currentTree = tree;
    this.currentFile = {
      reference: tree,
      content: this.currentFile?.content
    };

    if(this.viewer == FileViewer.pdf) {
      this.solutionService.getFileAsUrl(this.solution.id, tree)
      .pipe(takeUntil(this.unsubscribe))
      .subscribe(([url, fileName]) => {
        this.pdfEmbedUrl = this.sanitizer.bypassSecurityTrustResourceUrl(url + '#view=FitH');
      }, (error: any) => {
        if (BackendError.isBackendError(error)) {
          this.notificationService.showError(
            error.format(
              "Fehler beim Herunterladen der Lösung",
            )
          );
        } else {
          throw error;
        }
      });
    }

    if (this.viewer == FileViewer.sourceCode) {
      this.solutionService.getFile(this.solution.id, tree)
        .pipe(takeUntil(this.unsubscribe))
        .subscribe(content => {
          this.currentFile = {
            reference: tree,
            content: content
          };
        }, (error: any) => {
          if (BackendError.isBackendError(error)) {
            this.notificationService.showError(
              error.format(
                "Fehler beim Herunterladen der Lösung",
              )
            );
          } else {
            throw error;
          }
        });
    }
  }

  onDownloadTree(tree: FileTree) {
    this.solutionService.downloadFile(this.solution.id, tree)
    .pipe(takeUntil(this.unsubscribe))
    .subscribe(() => {}, (error: any) => {
      if (BackendError.isBackendError(error)) {
        this.notificationService.showError(
          error.format(
            "Fehler beim Herunterladen der Lösung",
          )
        );
      } else {
        throw error;
      }
    });
  }
}
