import { Component, OnInit, ViewChild } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { NotificationService } from 'app/shared/services/notification.service';
import { combineLatest } from 'rxjs';
import { flatMap, takeUntil, filter } from 'rxjs/operators';
import { User } from 'app/core/models/user';
import { Task } from 'app/features/tasks/models/task';
import { AuthService } from 'app/core/services/auth.service';
import { TaskPoolService } from '../../services/task-pool.service';
import { TaskPool } from '../../models/task-pool';
import { BackendError } from 'app/shared/models/backend-error';
import { NbDialogService } from '@nebular/theme';
import { DialogConfirmationComponent } from 'app/shared/dialogs/dialog-confirmation/dialog-confirmation.component';
import { ErrorContainerConfig } from 'app/shared/components/error-container/error-container.component';
import { TaskImportLogsComponent } from '../../components/task-import-logs/task-import-logs.component';
import { TaskPoolTasksComponent } from '../../components/task-pool-tasks/task-pool-tasks.component';
import { Breadcrumb } from 'app/shared/components/breadcrumbs/breadcrumbs.component';
import { SubatoPrincipal } from 'app/core/models/principal/subato-principal';
import { environment } from 'environments/environment';

@Component({
  selector: 'ngx-task-pool',
  templateUrl: './task-pool.component.html',
  styleUrls: ['./task-pool.component.scss']
})
export class TaskPoolComponent extends BaseComponent implements OnInit {
  @ViewChild(TaskImportLogsComponent, { static: false }) logsComponent!: TaskImportLogsComponent;
  @ViewChild(TaskPoolTasksComponent, { static: false }) tasksComponent!: TaskPoolTasksComponent;

  stefDocsUrl = environment.stefDocsUrl;

  taskPool: TaskPool;
  loading: boolean;
  principal: SubatoPrincipal;
  poolId: number;
  archivingTask: Task | null;
  deletingPool = false;
  syncing = false;

  error: BackendError;
  errorConfig: ErrorContainerConfig = {
    notFound: {
      message: 'Diesen Aufgaben-Pool konnten wir leider nicht finden.'
    }
  };

  breadcrumbs: Breadcrumb[] = [];

  constructor(
    private taskPoolService: TaskPoolService,
    private dialogService: NbDialogService,
    private router: Router,
    private authService: AuthService,
    private notificationService: NotificationService,
    private route: ActivatedRoute) {
    super();
  }

  buildBreadcrumbs(pool: TaskPool) {
    this.breadcrumbs = [
      {
        id: 'pools',
      },
      {
        id: 'pool',
        params: {
          pool_id: pool.id
        },
        ctx: pool
      },
    ];
  }

  ngOnInit(): void {
    this.loading = true;
    combineLatest(this.route.paramMap, this.authService.principal$)
      .pipe(
        takeUntil(this.unsubscribe),
        flatMap(([params, principal]) => {
          this.principal = principal;
          this.poolId = Number(params.get("pool_id"));
          return this.taskPoolService.getById(this.poolId);
        })
      )
      .subscribe(taskPool => {
        this.taskPool = taskPool;
        this.buildBreadcrumbs(taskPool);
        this.loading = false;
      }, error => {
        if (BackendError.isBackendError(error)) {
          this.error = error;
          this.loading = false;
        } else {
          throw error;
        }
      });
  }

  onTaskUploaded(task: Task) {
    this.tasksComponent.refresh();
  }

  deletePool() {
    this.dialogService.open(DialogConfirmationComponent, {
      context: {
        title: `Löschung von Aufgaben-Pool: ${this.taskPool.name}`,
        text: `Bist du wirklich sicher, dass du den Pool '${this.taskPool.name}' löschen willst? Alle verknüpften Aufgaben, Lösungen etc. werden damit permanent gelöscht.`,
        actionText: 'Löschen'
      }
    }).onClose.pipe(
      takeUntil(this.unsubscribe),
      filter(result => result && result.confirmed),
      flatMap(() => {
        this.deletingPool = true;
        return this.taskPoolService.delete(this.taskPool.id);
      })
    ).subscribe(() => {
      this.deletingPool = false;
      this.router.navigate(['task_pools']);
      this.notificationService.showSuccess("Aufgaben-Pool erfolgreich gelöscht.");
    }, error => {
      this.deletingPool = false;
      if (BackendError.isBackendError(error)) {
        this.notificationService.showError(
          error.format(
            "Fehler beim Löschen des Aufgaben-Pools",
          )
        );
      } else {
        throw error;
      }
    })
  }

  sync() {
    this.syncing = true;
    return this.taskPoolService.sync(this.poolId)
    .pipe(
      takeUntil(this.unsubscribe)
    )
    .subscribe(result => {
      this.syncing = false;
      if(result.problems == 0) {
        this.notificationService.showSuccess("Synchronisation erfolgreich");
      } else {
        this.notificationService.showWarning("Bei der Synchronisation sind Probleme aufgetreten");
      }

      this.tasksComponent.refresh();
      this.logsComponent.refresh();
    }, error => {
      this.syncing = false;

      if (BackendError.isBackendError(error)) {
        this.notificationService.showError(
          error.format(
            "Fehler bei Synchronisation",
          )
        );
      } else {
        throw error;
      }
    })
  }
}
