package de.hsrm.sls.subato.auth.masking;

import de.hsrm.sls.subato.auth.principal.PrincipalService;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.shared.error.ErrorResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Component;

import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.util.*;

@Component
public class MaskManager {
    @Autowired
    private List<Masker> maskers;

    @Autowired
    private PrincipalService principalService;

    public <T> T mask(T obj) {
        assert obj != null;

        if(obj instanceof ErrorResponse) {
            return obj;
        }

        var principal = principalService.getPrincipal();

        if(Page.class.isAssignableFrom(obj.getClass())) {
            var page = (Page) obj;
            var firstEle = page.getContent().stream().findFirst().orElse(null);
            if(firstEle != null) {
                for(var ele : page.getContent()) {
                    mask(ele, principal);
                }
            }
        } else {
            mask(obj, principal);
        }

        return obj;
    }

    private List<Masker> getSupported(Object obj) {
        var m = new ArrayList<Masker>();
        for(var masker : maskers) {
            if(masker.supports(obj.getClass())) {
                m.add(masker);
            }
        }
        return m;
    }

    private void mask(Object obj, SubatoPrincipal principal) {
        if(obj == null) {
            return;
        }

        if(isCollection(obj)) {
            var col = (Collection) obj;
            col.forEach(e -> mask(e, principal));
        } else {
            if (!obj.getClass().getPackageName().startsWith("de.hsrm.sls.subato")) {
                return;
            }

            var maskers = getSupported(obj);
            maskers.forEach(m -> m.mask(obj, principal));

            var properties = beanProperties(obj);

            for (var f : properties.values()) {
                if (f == null) {
                    continue;
                }

                mask(f, principal);
            }
        }
    }

    public static boolean isCollection(Object ob) {
        return ob instanceof Collection;
    }

    // https://stackoverflow.com/questions/8524011/java-reflection-how-can-i-get-the-all-getter-methods-of-a-java-class-and-invoke
    public static Map<String, Object> beanProperties(Object bean) {
        try {
            Map<String, Object> map = new HashMap<>();
            Arrays.asList(Introspector.getBeanInfo(bean.getClass(), Object.class)
                            .getPropertyDescriptors())
                    .stream()
                    // filter out properties with setters only
                    .filter(pd -> Objects.nonNull(pd.getReadMethod()))
                    .forEach(pd -> { // invoke method to get value
                        try {
                            Object value = pd.getReadMethod().invoke(bean);
                            if (value != null) {
                                map.put(pd.getName(), value);
                            }
                        } catch (Exception e) {
                            // add proper error handling here
                        }
                    });
            return map;
        } catch (IntrospectionException e) {
            // and here, too
            return Collections.emptyMap();
        }
    }
}
