package de.hsrm.sls.subato.courses.unit;

import de.hsrm.sls.subato.auth.policy.CheckPolicyRequest;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.courses.CoursesAction;
import de.hsrm.sls.subato.courses.event.CourseUnitEventMapper;
import de.hsrm.sls.subato.courses.event.CourseUnitEventSummaryDto;
import de.hsrm.sls.subato.user.UserDetailDto;
import de.hsrm.sls.subato.user.UserMapper;
import de.hsrm.sls.subato.user.UserService;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;

import java.util.List;
import java.util.stream.Collectors;

import static org.springframework.http.HttpStatus.BAD_REQUEST;
import static org.springframework.http.HttpStatus.NOT_FOUND;

@RestController
@Tag(name = "Kursverwaltung")
public class CourseUnitController {
    @Autowired
    CourseUnitService courseUnitService;

    @Autowired
    private UserService userService;

    @Autowired
    CourseUnitMapper courseUnitMapper;

    @Autowired
    CourseUnitEventMapper courseUnitEventMapper;

    @Autowired
    UserMapper userMapper;

    @Autowired
    private PolicyEvaluator policy;

    @GetMapping("/course_unit/{id}")
    CourseUnitDto getOne(@PathVariable Long id) {
        try {
            var courseUnit = courseUnitService.findById(id);
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(courseUnit.getCourse(), CoursesAction.COURSE_VIEW));

            return courseUnitMapper.map(courseUnit);
        } catch(CourseUnitNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, CourseUnitNotFoundException.REASON);
        }
    }

    @GetMapping("/course_unit/{id}/events")
    List<CourseUnitEventSummaryDto> getEvents(@PathVariable Long id) {
        try {
            var courseUnit = courseUnitService.findById(id);
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(courseUnit.getCourse(), CoursesAction.COURSE_VIEW));

            var events = courseUnitService.getEvents(courseUnit);
            return events.stream().map(e -> courseUnitEventMapper.mapSummary(e)).toList();
        } catch(CourseUnitNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, CourseUnitNotFoundException.REASON);
        }
    }

    @GetMapping("/course_unit/{id}/participants")
    List<UserDetailDto> getParticipants(@PathVariable Long id) {
        try {
            var courseUnit = courseUnitService.findById(id);
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(courseUnit.getCourse(), CoursesAction.COURSE_VIEW_PARTICIPANTS));

            var participants = courseUnitService.getParticipants(courseUnit);
            return participants.stream().map(u -> userMapper.mapDetail(u)).collect(Collectors.toList());
        } catch(CourseUnitNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, CourseUnitNotFoundException.REASON);
        }
    }

    @PutMapping("/course_unit/{id}/participants/{userId}")
    public void addParticipant(@PathVariable Long id, @PathVariable Long userId) {
        try {
            var courseUnit = courseUnitService.findById(id);
            var user = userService.findById(userId);
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(courseUnit, CoursesAction.COURSE_UNIT_UPDATE));

            courseUnitService.addParticipant(courseUnit, user);
        } catch(CourseUnitNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, CourseUnitNotFoundException.REASON);
        } catch(UserIsAlreadyParticipantException ex) {
            throw new ResponseStatusException(BAD_REQUEST, UserIsAlreadyParticipantException.REASON);
        }
    }

    @DeleteMapping("/course_unit/{id}/participants/{userId}")
    public void removeParticipant(@PathVariable Long id, @PathVariable Long userId) {
        try {
            var courseUnit = courseUnitService.findById(id);
            var user = userService.findById(userId);
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(courseUnit, CoursesAction.COURSE_UNIT_UPDATE));

            courseUnitService.removeParticipant(courseUnit, user);
        } catch(CourseUnitNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, CourseUnitNotFoundException.REASON);
        } catch(UserIsNotAParticipantException ex) {
            throw new ResponseStatusException(NOT_FOUND, UserIsNotAParticipantException.REASON);
        }
    }
}
