package de.hsrm.sls.subato.search;

import de.hsrm.sls.subato.auth.policy.CheckPolicyRequest;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.courses.CoursesAction;
import de.hsrm.sls.subato.auth.policy.exercises.ExercisesAction;
import de.hsrm.sls.subato.courses.course.Course;
import de.hsrm.sls.subato.courses.course.CourseMapper;
import de.hsrm.sls.subato.courses.course.CourseNotFoundException;
import de.hsrm.sls.subato.courses.course.CourseService;
import de.hsrm.sls.subato.exercises.exercise.Exercise;
import de.hsrm.sls.subato.exercises.exercise.ExerciseMapper;
import de.hsrm.sls.subato.exercises.taskinstance.TaskInstance;
import de.hsrm.sls.subato.exercises.taskinstance.TaskInstanceMapper;
import de.hsrm.sls.subato.search.item.CourseParticipantItem;
import de.hsrm.sls.subato.search.item.ExerciseItem;
import de.hsrm.sls.subato.search.item.Item;
import de.hsrm.sls.subato.search.item.TaskInstanceItem;
import de.hsrm.sls.subato.user.User;
import de.hsrm.sls.subato.user.UserMapper;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.server.ResponseStatusException;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import static org.springframework.http.HttpStatus.BAD_REQUEST;


@RestController
@Tag(name = "Querschnitt")
public class SearchController {
    @Autowired
    SearchService searchService;

    @Autowired
    CourseMapper courseMapper;

    @Autowired
    TaskInstanceMapper taskInstanceMapper;

    @Autowired
    ExerciseMapper exerciseMapper;

    @Autowired
    UserMapper userMapper;

    @Autowired
    CourseService courseService;

    @Autowired
    PolicyEvaluator policy;

    static String exerciseGroup(Exercise exercise) {
        return exercise.getName();
    }

    List<TaskInstanceItem> getTaskItems(List<TaskInstance> tasks) {
        return tasks.stream()
            .map(t-> {
                var dto = taskInstanceMapper.mapDetail(t);
                return new TaskInstanceItem(dto, exerciseGroup(t.getExercise()));
            }).collect(Collectors.toList());
    }

    List<ExerciseItem> getExerciseItems(List<Exercise> exercises) {
        return exercises.stream().map(m-> {
            var dto = exerciseMapper.map(m);
            return new ExerciseItem(dto, "Übungen");
        }).collect(Collectors.toList());
    }

    List<CourseParticipantItem> getCourseParticipantItems(List<User> users, Course course) {
        return users.stream().map(m-> {
            var dto = userMapper.mapDetail(m);
            return new CourseParticipantItem(dto, course.getId(), "Teilnehmer");
        }).collect(Collectors.toList());
    }

    @GetMapping("/search")
    @Operation(summary = "Querschnittliche Suche innerhalb eines Kurses über alle Aspekte hinweg (Aufgaben, Kursteilnehmer, ...)")
    SearchResult search(@RequestParam("courseId") Long courseId, @RequestParam("q") String query) {
        var items = new ArrayList<Item>();

        try {
            var course = courseService.findById(courseId);

            var exercises = searchService.searchExercises(course, query);
            items.addAll(getExerciseItems(exercises));

            var taskInstances = searchService.searchTaskInstances(course, query)
                    .stream()
                    .filter(taskInstance -> policy.check(CheckPolicyRequest.fromPrincipal(taskInstance.getExercise(), ExercisesAction.EXERCISE_VIEW)))
                    .collect(Collectors.toList());
            items.addAll(getTaskItems(taskInstances));

            if(policy.check(CheckPolicyRequest.fromPrincipal(course, CoursesAction.COURSE_VIEW_PARTICIPANTS))) {
                var participants = searchService.searchCourseParticipants(query, course);
                items.addAll(getCourseParticipantItems(participants, course));
            }
        } catch(CourseNotFoundException ex) {
            throw new ResponseStatusException(BAD_REQUEST, "Der Kurs existiert nicht");
        }

        return new SearchResult(query, items);
    }
}
