package de.hsrm.sls.subato.solutions.evaluation.request;

import de.hsrm.sls.subato.shared.fs.FileTree;
import de.hsrm.sls.subato.tasks.stef.file.FileRef;

/**
 *  Matches a source file within the submitted directory by skipping the configured prefix.
 *
 *  EXAMPLE 1:
 *  prefix: "src"
 *  template path: panitz/name/First.java
 *  solution path:
 *
 *  - solutions
 *    - 17 (root)
 *      - src <--- ignore this
 *          - panitz
 *              - name
 *                  - First.java
 *
 *  EXAMPLE 2:
 *  prefix: "src"
 *  template path: First.java
 *  solution path:
 *
 *  - solutions
 *      - 17 (root)
 *          - src <--- ignore this
 *              - First.java
 *
 *  EXAMPLE 3:
 *  prefix: "*"
 *  template path: panitz/name/First.java
 *  solution path:
 *
 *  - solutions
 *    - 17 (root)
 *      - ABC <--- ignore this
 *          - panitz
 *              - name
 *                  - First.java
 *
 *  EXAMPLE 4:
 *  prefix: "* / src" <-- (without whitespace, inserted because of javadoc parser)
 *  template path: panitz/name/First.java
 *  solution path:
 *
 *  - solutions
 *    - 17 (root)
 *      - ABC <--- ignore this
 *          - src <--- this too
 *              - panitz
 *                  - name
 *                      - First.java
 */
public class PrefixMatcher implements FileMatcher {

    /**
     * path, where each segment consists of constants (e.g. "src") or *.
     *
     * * matches any value in the current segment (lazy, not greedy), see example 4
     */
    private String prefix;

    public PrefixMatcher(String prefix) {
        this.prefix = prefix;
    }

    @Override
    public FileTree match(FileTree solutionTree, FileRef templateFile) {
        return solutionTree.find(i -> i.isSrc() && matches(i, templateFile));
    }

    private boolean matches(FileTree item, FileRef templateFile) {
        var prefixSegments = prefix.split("/");

        var segments = item.getPath().split("/", prefixSegments.length + 1);
        if(segments.length != prefixSegments.length + 1) {
            return false;
        }

        for(var i = 0; i < prefixSegments.length; i++) {
            var prefixSegment = prefixSegments[i];
            if(!prefixSegment.equals("*") && !prefixSegment.equals(segments[i])) {
                return false;
            }
        }

        var remaining = segments[segments.length - 1];
        return remaining.equals(templateFile.getPath());
    }
}
