package de.hsrm.sls.subato.solutions.solution;

import de.hsrm.sls.subato.auth.policy.CheckPolicyRequest;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.solutions.SolutionsAction;
import de.hsrm.sls.subato.exercises.exercise.ExerciseNotFoundException;
import de.hsrm.sls.subato.exercises.taskinstance.TaskInstanceNotFoundException;
import de.hsrm.sls.subato.shared.FileResponseBuilder;
import de.hsrm.sls.subato.shared.fs.FileTreeMapper;
import de.hsrm.sls.subato.shared.helper.ResourceHelper;
import de.hsrm.sls.subato.solutions.evaluation.EvaluationFailedException;
import de.hsrm.sls.subato.solutions.evaluation.InvalidSolutionException;
import de.hsrm.sls.subato.solutions.evaluation.SolutionEvaluationService;
import de.hsrm.sls.subato.solutions.evaluation.result.EvalResultDetailDto;
import de.hsrm.sls.subato.solutions.evaluation.result.EvalResultMapper;
import de.hsrm.sls.subato.solutions.submit.InvalidModeException;
import de.hsrm.sls.subato.solutions.submit.SubmitFileSolutionDto;
import de.hsrm.sls.subato.solutions.submit.SubmitGate;
import de.hsrm.sls.subato.solutions.submit.SubmitInlineSolution;
import de.hsrm.sls.subato.tasks.task.TaskNotFoundException;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.Resource;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.web.PageableDefault;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.server.ResponseStatusException;

import java.io.IOException;
import java.util.Optional;
import java.util.UUID;

import static org.springframework.http.HttpStatus.*;

@RestController
@Tag(name = "Lösungsverwaltung")
public class SolutionController {
    @Autowired
    private SolutionService solutionService;

    @Autowired
    private SubmitGate submitGate;

    @Autowired
    private SolutionMapper solutionMapper;

    @Autowired
    private EvalResultMapper evalResultMapper;

    @Autowired
    private FileTreeMapper fileTreeMapper;

    @Autowired
    private SolutionEvaluationService solutionEvaluationService;

    @Autowired
    private PolicyEvaluator policy;

    @Autowired
    private FileResponseBuilder fileResponseBuilder;

    private static final String solutionSubmitDescription = """
    Die Abgabe von Lösungen kann in zwei Modi erfolgen, weshalb dieser Endpunkt zwei verschiedene Eingaben entgegen nehmen kann:  \n
    1.) Abgabe einer Datei bzw. eines zip-Archives (multipart/form-data)\n
    2.) Abgabe von Inhalt einer Datei als String (application/json)\n
    \n
    Welcher Modus verwendet werden kann, ergibt sich aus der Aufgabe. Siehe `SubmissionMode` in der STEF Spezifikation für weitere Informationen.
    \n
    Die Angabe von `lateSubmit` hat erst eine Bedeutung, wenn die Abgabefrist erreicht ist. Ist dann lateSubmit = `false`, wird die Lösung nicht mehr einer Abgabe zugeordnet. Wenn lateSubmit = `true`, wird die Lösung als verspätet gekennzeichnet und der Abgabe hinzugefügt.
    `comment` ist Optional und kann angegeben werden um einen Kommentar zur Lösung hinzuzfügen.
    
    Bei der Abgabe muss zu Forschungszwecken auch angegeben werden, mit welchem Client (Weboberfläche oder Eclipse-Plugin) abgegeben wurde.
    """;

    private static final String solutionSubmitSummary = "Speicherung und automatische Auswertung von Lösungen (wenn für die Aufgabe konfiguriert).";

    @GetMapping("/solution/{id}")
    SolutionDetailDto getOne(@PathVariable Long id) {
        try  {
            var solution = solutionService.findById(id);
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(solution, SolutionsAction.SOLUTION_VIEW));

            var tree = solutionService.getTree(solution);
            var dto = solutionMapper.mapDetail(solution);
            dto.setTree(fileTreeMapper.map(tree));
            return dto;
        } catch(SolutionNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, "Die Lösung existiert nicht");
        }
    }

    @GetMapping("/solution")
    @PreAuthorize("isAuthenticated()")
    @Operation(summary = "Alle sichtbaren Lösungen im Kontext des angemeldeten Benutzers")
    Page<SolutionSummaryDto> getAll(
            @PageableDefault Pageable page,
            @RequestParam Optional<Long> exerciseId,
            @RequestParam Optional<Long> taskId,
            @RequestParam Optional<Long> userId,
            @RequestParam Optional<Long> submissionId,
            @RequestParam Optional<Boolean> valid) {
        var filter = new SolutionFilter()
                .exerciseId(exerciseId.orElse(null))
                .taskId(taskId.orElse(null))
                .submittedById(userId.orElse(null))
                .submissionId(submissionId.orElse(null))
                .valid(valid.orElse(null));
        var solutions = solutionService.findAll(page, filter, SolutionsAction.SOLUTION_VIEW);

        return solutions.map(solution -> solutionMapper.mapSummary(solution));
    }

    @GetMapping("/solution/{id}/file/{uuid}")
    @Operation(summary = "Zugriff auf Dateien in der Lösungen, ermöglicht Browsing von komplexeren Lösungen die einen Verzeichnisbaum beinhalten")
    ResponseEntity<Resource> getFile(@PathVariable Long id, @PathVariable UUID uuid, @RequestParam Optional<Boolean> download) throws IOException {
        try {
            var solution = solutionService.findById(id);
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(solution, SolutionsAction.SOLUTION_VIEW));

            var file = solutionService.getFile(solution, uuid);
            var resource = ResourceHelper.load(file);

            return fileResponseBuilder.build(resource, download.isPresent() && download.get());
        } catch(SolutionNotFoundException ex) {
            throw new ResponseStatusException(BAD_REQUEST, "Die Lösung existiert nicht");
        }
    }

    @PostMapping(path ="/solution/submit", consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    @Operation(summary = solutionSubmitSummary, description = solutionSubmitDescription)
    SolutionDetailDto submit(
            @Parameter(description = "Lösungsdatei (Datei oder zip-Archiv)", required = false)
            @RequestPart("file")
            MultipartFile file,
            @Parameter(description = "Metadaten wie Verweise auf Aufgabe und Übung, die für die Zuordnung der Lösung erforderlich sind. Das DTO heißt `SubmitFileSolutionDto`, hier wird leider das Schema nicht angezeigt aber man kann es unten in der Auflistung finden.")
            @RequestPart("solution")
            SubmitFileSolutionDto dto) {
        try {
            var solution = submitGate.submit(file, dto);
            return solutionMapper.mapDetail(solution);
        } catch(TaskInstanceNotFoundException ex) {
            throw new ResponseStatusException(BAD_REQUEST, TaskInstanceNotFoundException.REASON);
        } catch(InvalidModeException ex) {
            throw new ResponseStatusException(BAD_REQUEST, InvalidModeException.REASON);
        } catch(EvaluationFailedException ex) {
            throw new ResponseStatusException(INTERNAL_SERVER_ERROR, EvaluationFailedException.REASON, ex);
        } catch(InvalidSolutionException ex) {
            throw new ResponseStatusException(BAD_REQUEST, "%s: %s".formatted(InvalidSolutionException.REASON, ex.getMessage()));
        }
    }

    @PostMapping(path ="/solution/submit", consumes = MediaType.APPLICATION_JSON_VALUE)
    @Operation(summary = solutionSubmitSummary, description = solutionSubmitDescription)
    SolutionDetailDto submit(
            @Parameter(description = "Enthält die Lösung selbst und Metadaten, wie Verweise auf Aufgabe und Übung, die für die Zuordnung der Lösung erforderlich sind.")
            @RequestBody SubmitInlineSolution dto) {
        try {
            var solution = submitGate.submit(dto);
            return solutionMapper.mapDetail(solution);
        } catch(TaskNotFoundException ex) {
            throw new ResponseStatusException(BAD_REQUEST, TaskNotFoundException.REASON);
        } catch(ExerciseNotFoundException ex) {
            throw new ResponseStatusException(BAD_REQUEST, ExerciseNotFoundException.REASON);
        } catch(InvalidModeException ex) {
            throw new ResponseStatusException(BAD_REQUEST, InvalidModeException.REASON);
        } catch(EvaluationFailedException ex) {
            throw new ResponseStatusException(INTERNAL_SERVER_ERROR, EvaluationFailedException.REASON, ex);
        }
    }

    @PostMapping("/solution/{id}/acceptancetest")
    @Operation(summary = "Ausführung von Akzeptanztests durch Lehrende, deren Testergebnisse nicht gespeichert sondern nur zurückgegeben werden")
    EvalResultDetailDto testSolution(@PathVariable Long id) {
        try {
            var solution = solutionService.findById(id);
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(solution, SolutionsAction.SOLUTION_RUN_ACCEPTANCE_TESTS));

            return evalResultMapper.mapDetail(solutionEvaluationService.test(solution, true));
        } catch(SolutionNotFoundException ex) {
            throw new ResponseStatusException(BAD_REQUEST, "Die Lösung existiert nicht");
        } catch(EvaluationFailedException ex) {
            throw new ResponseStatusException(INTERNAL_SERVER_ERROR, EvaluationFailedException.REASON);
        }
    }
}
