package de.hsrm.sls.subato.tasks.imp;

import de.hsrm.sls.subato.shared.fs.FileContent;
import de.hsrm.sls.subato.shared.fs.TmpDirProvider;
import de.hsrm.sls.subato.tasks.imp.log.LogType;
import de.hsrm.sls.subato.tasks.imp.log.TaskImportLogService;
import de.hsrm.sls.subato.tasks.stef.file.FileType;
import de.hsrm.sls.subato.tasks.stef.parser.InvalidSpecException;
import de.hsrm.sls.subato.tasks.stef.StefSpec;
import de.hsrm.sls.subato.tasks.stef.parser.SpecParser;
import de.hsrm.sls.subato.tasks.pool.TaskPool;
import de.hsrm.sls.subato.tasks.pool.TaskPoolService;
import de.hsrm.sls.subato.tasks.task.SubmissionMode;
import de.hsrm.sls.subato.tasks.task.Task;
import de.hsrm.sls.subato.tasks.task.TaskContentRepository;
import de.hsrm.sls.subato.tasks.task.TaskRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.FileSystemUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.util.ArrayList;
import java.util.stream.Collectors;

/***
 * Responsible for importing or updating a {@link Task}, represented by a {@link StefSpec}, into the database
 */
@Service
public class TaskImportService {
    private static Logger logger = LoggerFactory.getLogger(TaskImportService.class);

    @Autowired
    private TaskRepository taskRepository;

    @Autowired
    private SpecParser specService;

    @Autowired
    private TaskPoolService taskPoolService;

    @Autowired
    private TaskImportLogService importLogService;

    @Autowired
    private TaskContentRepository taskContentRepository;

    @Autowired
    private TmpDirProvider tmpDirProvider;

    /**
     * checks whether there already is a task in the pool with the id given in the spec or not.
     * if not, a new task will be created and linked to the pool.
     * in both cases the task will be updated to reflect the metadata of the spec
     * @param spec
     * @param pool
     * @return created or updated task
     */
    private Task getOrCreateTask(StefSpec spec, TaskPool pool) {
        var task = taskRepository.findByIdNameAndPoolId(spec.getMeta().getId(), pool.getId());
        if(task == null) {
            task = new Task();
            task.setPool(pool);
            task.setIdName(spec.getMeta().getId());
            pool.getTasks().add(task);
        }
        
        task.setName(spec.getMeta().getName());
        task.setDescription(spec.getMeta().getDescription());
        task.setAttempts(spec.getMeta().getAttempts());
        task.setContent(spec.getText());
        task.setEvaluator(spec.getMeta().getEvaluator());
        task.setLanguage(spec.getMeta().getLanguage());
        task.setDefaultSubmissionMode(spec.getMeta().getSubmissionModes().getDefaultMode());
        task.setSupportsFileSubmissionMode(spec.getMeta().getSubmissionModes().getModes().contains(SubmissionMode.FILE));
        task.setSupportsTextSubmissionMode(spec.getMeta().getSubmissionModes().getModes().contains(SubmissionMode.TEXT));
        task.setArchived(false);

        if(spec.getMeta().getLegacyId() != null) {
            var existingLegacyTask = taskRepository.findBySourceId(spec.getMeta().getLegacyId()).orElse(null);
            if(existingLegacyTask != null && task.getId() != existingLegacyTask.getId()) {
                // prevent exception because of unique constraint on sourceId
                logger.warn("Ignoring sourceId because there already is a task with sourceId=%s".formatted(spec.getMeta().getLegacyId()));
            } else {
                task.setSourceId(spec.getMeta().getLegacyId());
            }
        }

        var evaluationPossible = spec.getMeta().getEvaluator() != null && spec.getMeta().getFiles().stream().anyMatch(f -> f.getType() == FileType.TEST);
        task.setEvaluationPossible(evaluationPossible);
        var isAcceptanceTestAvailable = spec.getMeta().getEvaluator() != null && spec.getMeta().getFiles().stream().anyMatch(f -> f.getType() == FileType.ACCEPTANCETEST);
        task.setAcceptanceTestAvailable(isAcceptanceTestAvailable);

        task = taskRepository.save(task);
        return task;
    }

    /**
     * parses the files represented by the given directory as STEF spec
     * and imports it upon success as a new task
     * or updates an existing task, if it already exists.
     *
     * the id field in the meta.xml file of the spec determines, if a task already exists
     *
     * @param pool import target
     * @param dir directory, representing a STEF spec
     * @return created or updated task
     * @throws TaskImportException
     * @throws InvalidSpecException
     */
    private Task importTask(TaskPool pool, File dir) throws TaskImportException {
        // try to parse spec...
        var spec = specService.parseSpec(dir.toPath().toString());

        try {
            // spec is ok, now update task metadata in db
            var task = getOrCreateTask(spec, pool);

            // copy the tmp dir
            taskContentRepository.store(new ImportSpecRequest(spec, task), dir);

            return task;
        } catch(Exception ex) {
            throw new TaskImportException(ex);
        }
    }

    /**
     * @see #importTask(TaskPool, File)
     */
    @Transactional
    public Task importTask(TaskPool pool, MultipartFile file) {
        var tmpPath = tmpDirProvider.createTmpPath();
        var tmpDir = tmpPath.toFile();

        try {
            var content = FileContent.fromMultipartFile(file);
            content.saveTo(tmpDir, true);

            return importTask(pool, tmpPath.toFile());
        } finally {
            // delete tmp dir
            FileSystemUtils.deleteRecursively(tmpDir);
        }
    }

    /**
     * Batch-imports multiple tasks at once from the given directory
     * and deletes tasks which are in the pool but not present in the root directory.
     * Each task needs to be in its own folder within the root directory.
     *
     * @see #importTask(TaskPool, File)
     */
    @Transactional
    public TaskImportResult importTasks(TaskPool pool, File rootDir) {
        if(!rootDir.exists()) {
            throw new TaskImportException("Directory %s does not exist".formatted(rootDir.getAbsolutePath()));
        }

        var currentTasks = pool.getTasks().stream().filter(p -> !p.isArchived()).collect(Collectors.toList());
        var importedTasks = new ArrayList<Task>();
        var problems = 0L;
        var totalTaskDirectories = 0L;
        
        logger.info("Importing tasks in %s for pool %s".formatted(rootDir.getAbsolutePath(), pool.getName()));
        for(var dir : rootDir.listFiles()) {
            if(!dir.isDirectory()) {
                continue;
            }

            if(dir.isHidden()) {
                continue;
            }

            totalTaskDirectories += 1;

            try {
                var task = importTask(pool, dir);
                importedTasks.add(task);

                logger.info("Imported task %s".formatted(task.getIdName()));
            } catch(InvalidSpecException ex) {
                logger.info("Format of task %s is invalid: %s".formatted(dir.getName(), ex.getMessage()));
                importLogService.log(pool, LogType.PROBLEM, "%s: %s. %s".formatted(dir.getName(), InvalidSpecException.REASON, ex.getMessage()));
                problems += 1;
            } catch(TaskImportException ex) {
                logger.error("Failed to import task %s".formatted(dir.getName()), ex);
                importLogService.log(pool, LogType.PROBLEM, "%s: %s".formatted(dir.getName(), TaskImportException.REASON));
                problems += 1;
            }
        }
        
        var missingTasks = currentTasks.stream()
                .filter(currentTask -> !importedTasks.stream().anyMatch(importedTask -> importedTask.getId() == currentTask.getId()))
                .collect(Collectors.toList());
        
        logger.info("Imported %s tasks for pool %s, %s tasks were missing in the root directory. Will be archived now...".formatted(importedTasks.size(), pool.getName(), missingTasks.size()));
        for(var missingTask : missingTasks) {
            missingTask.setArchived(true);
            missingTask = taskRepository.save(missingTask);

            logger.info("Archived task %s".formatted(missingTask.getIdName()));
            importLogService.log(pool, LogType.INFO, "Aufgabe %s archiviert".formatted(missingTask.getIdName()));
        }

        importLogService.log(pool, LogType.INFO, "Import durchgeführt. %s Aufgaben importiert/aktualisiert, %s Aufgaben archiviert".formatted(importedTasks.size(), missingTasks.size()));
        logger.info("Done importing tasks for pool %s".formatted(pool.getName()));
        return new TaskImportResult(totalTaskDirectories, importedTasks.size(), missingTasks.size(), problems);
    }

    /**
     * @see #importTasks(TaskPool, File) 
     */
    @Transactional
    public TaskImportResult importTasks(long poolId, File rootDir) {
        var pool = taskPoolService.findById(poolId);
        return importTasks(pool, rootDir);
    }
}
