package de.hsrm.sls.subato.tasks.task;

import de.hsrm.sls.subato.solutions.solution.Solution;
import de.hsrm.sls.subato.solutions.evaluation.evaluator.Evaluator;
import de.hsrm.sls.subato.tasks.feedback.TaskFeedback;
import de.hsrm.sls.subato.tasks.feedback.TaskFeedbackStats;
import de.hsrm.sls.subato.tasks.lang.Language;
import de.hsrm.sls.subato.exercises.taskinstance.TaskInstance;
import de.hsrm.sls.subato.tasks.pool.TaskPool;

import javax.persistence.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;

/**
 * represents a task (STEF) in the database, serves primarily as a cache (and for linking to other entities)
 */
@Entity
@Table(name = "task", uniqueConstraints={
        // task ids are unique within a task pool
        @UniqueConstraint(columnNames = {"idName", "pool_id"})
})
public class Task {
    /**
     * unique across all tasks, just for internal use
     */
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private long id;

    /**
     * STEF
     */
    private String idName;

    /**
     * STEF
     */
    private String name;

    /**
     * STEF
     */
    @Column(length = 1000000)
    private String description;

    /**
     * STEF
     * maps to text/text.html
     */
    @Column(length = 1000000)
    private String content;

    /**
     * STEF
     */
    private Integer attempts;

    /**
     * actual occurences of this task in exercises
     */
    @OneToMany(mappedBy = "task", cascade = CascadeType.REMOVE)
    private List<TaskInstance> instances;

    @ManyToOne
    private TaskPool pool;

    /**
     * STEF
     * maps to subatoId
     */
    @Column(unique = true)
    private Long sourceId;

    /**
     * STEF
     * just for display purposes
     */
    @ManyToOne
    private Language language;

    /**
     * STEF
     * null if solutions are not evaluated automatically
     */
    @ManyToOne(cascade = { CascadeType.PERSIST, CascadeType.MERGE, CascadeType.REFRESH })
    private Evaluator evaluator;

    /**
     * determines if solutions for this task can be evaluated automatically
     */
    private boolean evaluationPossible;

    /**
     * determines if lecturers can run additional acceptance tests while checking/grading the solution
     */
    private boolean isAcceptanceTestAvailable;

    /**
     * STEF
     */
    private SubmissionMode defaultSubmissionMode;

    private boolean supportsFileSubmissionMode;

    private boolean supportsTextSubmissionMode;

    /**
     * feedback entries for this task
     */
    @OneToMany(mappedBy = "task", cascade = CascadeType.ALL, orphanRemoval = true)
    private List<TaskFeedback> feedback = new ArrayList<>();

    /**
     * flag for soft deletion.
     *
     * soft deleted tasks are not visible but archived in order to keep references from solutions.
     * useful if someone deletes a task accidentally .
     */
    private boolean archived;

    @OneToMany(mappedBy = "task", cascade = CascadeType.REMOVE)
    private List<Solution> solutions;

    public Task() { }

    public Long getSourceId() {
        return sourceId;
    }

    public void setSourceId(Long sourceId) {
        this.sourceId = sourceId;
    }

    public long getId() {
        return id;
    }

    public void setId(long id) {
        this.id = id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public List<TaskInstance> getInstances() {
        return instances;
    }

    public void setInstances(List<TaskInstance> instances) {
        this.instances = instances;
    }

    public TaskPool getPool() {
        return pool;
    }

    public void setPool(TaskPool pool) {
        this.pool = pool;
    }

    public String getContent() {
        return content;
    }

    public void setContent(String content) {
        this.content = content;
    }

    public Language getLanguage() {
        return language;
    }

    public void setLanguage(Language language) {
        this.language = language;
    }

    public Evaluator getEvaluator() {
        return evaluator;
    }

    public void setEvaluator(Evaluator evaluator) {
        this.evaluator = evaluator;
    }

    public String getIdName() {
        return idName;
    }

    public void setIdName(String idName) {
        this.idName = idName;
    }

    public List<TaskFeedback> getFeedback() {
        return feedback;
    }

    public void setFeedback(List<TaskFeedback> feedback) {
        this.feedback = feedback;
    }

    public Integer getAttempts() {
        return attempts;
    }

    public void setAttempts(Integer attempts) {
        this.attempts = attempts;
    }

    public SubmissionMode getDefaultSubmissionMode() {
        return defaultSubmissionMode;
    }

    public void setDefaultSubmissionMode(SubmissionMode submissionMode) {
        this.defaultSubmissionMode = submissionMode;
    }

    public boolean isArchived() {
        return archived;
    }

    public void setArchived(boolean archived) {
        this.archived = archived;
    }

    public List<Solution> getSolutions() {
        return solutions;
    }

    public void setSolutions(List<Solution> solutions) {
        this.solutions = solutions;
    }

    /**
     * calculates the average of complexity, comprehensibility feedback entries
     * and counts the number of unchecked entries
     */
    public TaskFeedbackStats getFeedbackStats() {
        if(feedback.stream().count() == 0) {
            return null;
        }

        var unread = feedback.stream().filter(f -> !f.isRead()).count();
        var avgComplexity = feedback.stream().map(f -> f.getComplexity()).reduce(0, (sum, c) -> sum + c);
        var avgComprehensibility = feedback.stream().map(f -> f.getComprehensibility()).reduce(0, (sum, c) -> sum + c);

        var stats = new TaskFeedbackStats();
        stats.setUnread(unread);
        stats.setAvgComplexity(avgComplexity);
        stats.setAvgComprehensibility(avgComprehensibility);
        return stats;
    }

    public boolean evaluationPossible() {
        return evaluationPossible;
    }

    public void setEvaluationPossible(boolean evaluationPossible) {
        this.evaluationPossible = evaluationPossible;
    }

    public boolean isAcceptanceTestAvailable() {
        return isAcceptanceTestAvailable;
    }

    public void setAcceptanceTestAvailable(boolean acceptanceTestAvailable) {
        isAcceptanceTestAvailable = acceptanceTestAvailable;
    }

    public boolean isSupportsFileSubmissionMode() {
        return supportsFileSubmissionMode;
    }

    public void setSupportsFileSubmissionMode(boolean supportsFileSubmissionMode) {
        this.supportsFileSubmissionMode = supportsFileSubmissionMode;
    }

    public boolean isSupportsTextSubmissionMode() {
        return supportsTextSubmissionMode;
    }

    public void setSupportsTextSubmissionMode(boolean supportsTextSubmissionMode) {
        this.supportsTextSubmissionMode = supportsTextSubmissionMode;
    }
    
    public boolean isSupportedByEclipsePlugin() {
        var supportedLanguages = new String[] { Language.JAVA };
        return language != null && Arrays.stream(supportedLanguages).anyMatch(langId -> language.getId().equals(langId));
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Task task = (Task) o;
        return id == task.id;
    }

    @Override
    public int hashCode() {
        return Objects.hash(id);
    }

    @PreRemove
    private void detachEvaluator() {
        evaluator = null;
    }
}
